/*+F***********************************************************************
*
* File name:	chunk.c
*
* Author:	Paul Lew, General Systems Group, Inc. Salem, NH
* Created at:	04/07/88  18:40 PM
* Last update:	10/09/92  02:43 PM  (Edition: 16)
*
*  Description:	This program  will  read stdin and   divide them  into
*		"chunks"  of  20000 bytes  and  feed each  chunk to  a
*		program.  The   program   to  be executed   should  be
*		specified by -c 'command arg'.
*
*		'%' character   in  'command arg'    will be   treated
*		specially, currently, %i will be replaced by the chunk
*		number (starting from 1).  %b will be replaced  by the
*		buffer size.
*	
* Environment:	SunOS 4.1.2
*		SunOS 4.0.3c
*		4.3 BSD Vax
*		4.2 BSD Unix Pyramid (OSx 4.0)
*
* Usage:	chunk [-H] [-b buffer_size] [-p part] [-c 'command arg']
*
* Update History:
*
*      Date		Description					By
*    --------	------------------------------------------------------	---
*    04/07/88	Initial version						Lew
*    05/19/88	Added -p options to execute command for selected parts	Lew
*    01/05/92	Modify -p options to handle range m-n			Lew
*    10/09/92	Clean up before distribution				Lew
*
* Routines included:
*
*	main		main program
*	set_cmd		prepare command string, handle % specially.
*	proc_opt	routine to set options, calls getopt()
*	usage		routine to display help message
*
* Build:	cc -o chunk chunk.c getopt.o
*
* Example: tar -cf - | compress | chunk -c 'btoa | mail -s "part %i/10" lew'
*
**-F**********************************************************************/
#include	<stdio.h>
#include	<ctype.h>
#define	MAXPARTS	2000
#define	EOS		'\0'
#define	YES		(1)
#define	NO		(0)
#define	when		break;case
#define	otherwise	break;default

char	*Version = "1.2  01/05/92  01:43 PM";
char	*Author = "Paul Lew (lew@gsg.com)";
int	Blocksize = 20000;		/* block size for each chunk */
int	Part_flag = NO;			/* if YES, -p specified */
char	Part [MAXPARTS];		/* each char flag one part */
char	Cmd [BUFSIZ+1];			/* command buffer */
char	*Cmdp = NULL;

extern	char	*malloc ();
extern	FILE	*popen ();

/*-------------------------------------------------------------04/08/88-+
|									|
|	   M a i n    P r o g r a m    S t a r t s    H e r e 		|
|									|
+----------------------------------------------------------------------*/
main (argc, argv)
int	argc;
char	**argv;
	{
	FILE	*outfp;
	char	*bufp;
	int	n;
	int	idx = 0;
	int	byte_count = 0;

	proc_opt (argc, argv);

	if ((bufp = malloc (Blocksize)) == NULL) {
		fprintf (stderr, "malloc %d bytes failed, aborted\n",
			 Blocksize);
		exit (1);
		}
	
	while ((n = fread (bufp, 1, Blocksize, stdin)) > 0) {
		byte_count += n;
		idx++;
		if (Cmdp == NULL) continue;	 /* estimate total chunks */
		set_cmd (Cmd, Cmdp, idx, n);
		if (Part_flag && idx < MAXPARTS && Part [idx] == 0) continue;
		if ((outfp = popen (Cmd, "w")) == NULL) {
			fprintf (stderr, "popen failed, cmd: %s, aborted\n",
				 Cmd);
			exit (1);
			}
		fwrite (bufp, 1, n, outfp);
		pclose (outfp);
		}
	if (Cmdp == NULL)
		printf ("Total %d chunks, %d bytes\n", idx, byte_count);
	}

/*-------------------------------------------------------------04/08/88-+
|									|
|	       set_cmd : prepare command to be executed			|
|									|
+----------------------------------------------------------------------*/
set_cmd (outp, inp, idx, n)
char	*outp;
char	*inp;
int	idx;				/* current chunk index */
int	n;				/* current block size */
	{
	char	c;
	char	intbuf[10];

	while ((c = *inp++) != EOS) {
		if (c == '%') {
			if ((c = *inp++) == EOS) break;
			switch (c) {
				case 'i':  sprintf (intbuf, "%d", idx);
					   goto num;
				case 'b':  sprintf (intbuf, "%d", n);
num:					   strcpy (outp, intbuf);
					   outp += strlen (intbuf);
					   continue;
				default:   break; /* output the char */
				}
			}
		*outp++ = c;
		}
	*outp = EOS;
	}

extern	char	*optarg;	/* points to start of the option */
extern	int	optind;		/* index of next arg to process	 */
/*------------------------------------------------------------07/12/84--+
|									|
|	   proc_opt : routine to process command options		|
|									|
+----------------------------------------------------------------------*/
proc_opt (argc, argv)
int	argc;
char	**argv;
	{
	register char	c;
	int		n;

	while ((c = getopt (argc, argv, "Hb:c:p:")) != EOF) {
		switch (c) {
		    case 'c':	Cmdp = optarg;
				break;
		    case 'b':	n = atoi (optarg);
				if (n > 0) Blocksize = n;
				break;
		    case 'p':	set_part (optarg);
				break;
		    case 'H':	
		    default:	usage (argv[0]);
				exit (1);
		    }
		}
	return (optind);		/* index to first real argument */
	}

/*-------------------------------------------------------------07/01/86-+
|									|
|		    usage : display help message			|
|									|
+----------------------------------------------------------------------*/
static usage (pname)
char	*pname;			/* program name */
	{
	printf ("%s  Version %s\n", pname, Version);
	printf ("%s\n", Author);
	printf ("Usage: %s [-H] [-b nn] [-p n] [-c 'command']\n", pname);
	printf ("where options:\n");
	printf ("   -H       get this help message\n");
	printf ("   -b nn    set chunk buffer to nn bytes [20000]\n");
	printf ("   -p xxx   do cmd only for part specified in string xxx:\n");
	printf ("            xxx may consist of a list of numbers or ranges\n");
	printf ("            for example: 20,30,45-90,17\n");
	printf ("   -c cmd   execute cmd foreach chunk\n");
	printf ("            if -c not given, print # of chunks will produced\n");
	printf ("%% in cmd:\n");
	printf ("  '%%i'  replaced by current chunk number (index)\n");
	printf ("  '%%b'  replaced by current chunk buffer size\n");
	}

/*-------------------------------------------------------------01/05/92-+
|									|
|		     set_part : set part number array			|
|									|
+----------------------------------------------------------------------*/
set_part (s)
char	*s;
	{
	unsigned	from, to, i;
	char		*p = s;
	char		c;

	while (*p) {
		c = *p;
		if (isspace(c) || c == ',') {
			p++; continue;
			}
		else if (isdigit(c)) {
			Part_flag = YES;
			from = to = atoi(p);
			while (isdigit(*p)) p++;
			if (*p == '-') {
				p++;
				if (isdigit(*p)) {
					to = atoi(p);
					while (isdigit(*p)) p++;
					}
				}
			for (i=from; i<=to; i++) {
				if (i >= MAXPARTS) {
					fprintf (stderr, "part %d too big\n",
						 i);
					exit (1);
					}
				Part [i] = 1;
				}
			}
		}
	}
